"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeoCoder = void 0;
const leaflet_1 = __importDefault(require("leaflet"));
const Geolonia = __importStar(require("@geolonia/normalize-japanese-addresses"));
const PopupContent_1 = __importDefault(require("./PopupContent"));
const Control_1 = __importDefault(require("./Control"));
class Utils {
    static createPin = (map, latlng, data) => {
        const marker = leaflet_1.default.marker(latlng, {
            alt: `${data.pref} ${data.city} ${data.town}`
        });
        const popup = leaflet_1.default.popup().setContent(new PopupContent_1.default({
            "都道府県": data.pref,
            "市区町村": data.city,
            "町名": data.town,
            "緯度": latlng.lat,
            "経度": latlng.lng
        }));
        marker.bindPopup(popup); // popupをmarkerに関連付ける
        marker.setLatLng(latlng); // markerに緯度経度を設定
        marker.addTo(map); // markerをmapに追加
        marker.openPopup(); // popupを開く 
    };
    static moveTo = (map, location) => {
        map.flyTo(location);
    };
}
;
class GeoCoder extends leaflet_1.default.Control {
    _div;
    constructor(options) {
        super(options);
    }
    onAdd = (map) => {
        this._div = leaflet_1.default.DomUtil.create("div", "leaflet-community-geocoder");
        this._div.appendChild(new Control_1.default(GeoCoder.on(map)));
        return this._div;
    };
    static on = (map) => {
        // 関数を返す
        return async (address) => {
            const result = await Geolonia.normalize(address);
            if (result.level >= 3) {
                // level 3 まで判別できた場合には、緯度経度情報も返ってくる
                // https://github.com/geolonia/normalize-japanese-addresses/pull/113
                const latlng = leaflet_1.default.latLng({
                    lat: result.lat || 35,
                    lng: result.lng || 135
                });
                Utils.moveTo(map, latlng);
                Utils.createPin(map, latlng, {
                    pref: result.pref,
                    city: result.city,
                    town: result.town
                });
            }
            else if (result.level == 2) {
                // level 2 までしか判別できなかった場合は、
                // 別途データベースに問い合わせ、大まかな位置情報を取得する
                const url = `https://geolonia.github.io/japanese-addresses/api/ja/${result.pref}/${result.city}.json`;
                fetch(url).then(response => response.json()).then(json => {
                    const destination = json[0];
                    const latlng = leaflet_1.default.latLng({
                        lat: parseFloat(destination.lat),
                        lng: parseFloat(destination.lng)
                    });
                    Utils.moveTo(map, latlng);
                    Utils.createPin(map, latlng, {
                        pref: result.pref,
                        city: result.city,
                        town: destination.town + destination.koaza
                    });
                });
            }
            else {
                alert("もう少し詳しい住所を入力してください");
            }
            return result;
        };
    };
}
exports.GeoCoder = GeoCoder;
//# sourceMappingURL=GeoCoder.js.map